library Counter;

{$MODE Delphi}

// Delphi 5 DLL-source (COUNTER.DLL)
// Defines a simple binary counter component for ProfiLab
// with adjustable number of output bits

uses
  Interfaces,
  SysUtils,
  LCLIntf,
  Classes,
  Controls,
  Dlg_CounterConfig in 'Dlg_CounterConfig.pas' {CounterConfigDlg};

{$R *.RES}

Const  Inputs = 2;  // number of inputs
//       Outputs = 8; // number of outputs made adjustable

       {INPUTS}
       CLK   = 0;   // index of input variable CLK
       RST   = 1;   // index of input variable RST

       {OUTPUTS}

       {USER}
       CLK_old = 0;          // index of user variable CLK_old
       RST_old = 1;          // index of user variable RST_old
       Count = 2;            // index of user variable COUNT
       InitialCount = 3;     // index of user variable INITIALCOUNT
       CounterBits = 4;      // index of user variable COUNTERBITS
       DLLIndex = 100;       // fixed index of DLL No. (set by ProfiLab)

Type TDLLParams = array[0..100] of extended;   //Type of ProfiLab DLL parameters
     PDLLParams = ^TDLLParams;                 // Pointer to ProfiLab DLL parameters

function NumInputs: Byte;
begin
  result:=Inputs; //Define FIXED number of component input pins
end;

{function NumOutputs: Byte; // Made adjustable with 'NumOutputsEx'
begin
  result:=Outputs; //Define FIXED number of component output pins
end;}

{function NumInputsEx(PUser: PDLLParams): Byte; //Just in case number of inputs need to be variable
begin
  result:=...; //Define VARIABLE number of component input pins
end;}

function NumOutputsEx(PUser: PDLLParams): Byte;
begin
   // Define VARIABLE number of component output pins

   // Return number of outputs depending on configuration (4,8,16 bit counter)
   // (Configuration stored in PUser^[CounterBits], so we need the extended function type)

   case round(PUser^[CounterBits]) of
   0: result:=4; //4 bit counter
   1: result:=8; //8 bit counter
   2: result:=16; //16 bit counter
   else result:=0 // Just to be complete
   end;

end;

Function InputName(Channel: Byte): ShortString; // Return name for each component input pin
begin
   case Channel of
   CLK: result:='CLK';   //  "CLK" (Clock)
   RST: result:='/RST';  //  "/RST" (NOT RESET)
   end;
end;

Function OutputName(Channel: Byte): ShortString; // Return name for each component output pin
begin
   result:='Q'+intToStr(Channel); //"Q0".."Q(n-1)" (Binary output) n=4,8,16
end;

Procedure SimStart(PInput,POutput,PUser: PDLLParams); //called when ProfiLab enters RUN mode
var i: Integer;
    NumBits: Integer;
begin
   NumBits:=NumOutputsEx(PUser); // 4,8,16
   PUser^[Count]:=PUser^[InitialCount]; //RESET COUNTER
   For i:=0 to NumBits-1 do
   begin
      if (round(PUser^[Count]) and (1 shl i))>0 then POutput^[i]:=5 else POutput[i]:=0; //Set binary outputs with initial COUNT
   end;
end;

Procedure SimStop(PInput,POutput,PUser: PDLLParams); //called when ProfiLab RUN mode is terminated
begin
  // nothing to be done
end;

Procedure Calculate(PInput,POutput,PUser: PDLLParams); //called regularly from ProfiLab
var i: Integer;
    NumBits: Integer;
    MaxCount: Word;
begin
   NumBits:=NumOutputsEx(PUser);
   Case NumBits of
   4: MaxCount:=$0F;
   8: MaxCount:=$FF;
   16: MaxCount:=$FFFF;
   end;

   if PInput^[RST]<2.5 then //check RST input HIGH or LOW
   begin
      if (not (PInput^[RST]>=2.5)) and (PUser^[RST_old]>2.5) then //check out falling edge at RST input
      begin
         PUser^[Count]:=0; //RESET COUNT
         For i:=0 to NumBits-1 do
         begin
             POutput[i]:=0; //Set binary outputs with COUNT=0
         end;
      end;
      exit;
   end;
   PUser^[RST_old]:=PInput^[RST]; //Remember RST status for next call

   if PInput^[CLK]>2.5 then //check CLK input HIGH or LOW
   begin
      if (PInput^[CLK]>2.5) and not(PUser^[CLK_old]>2.5) then //check out rising edge at CLK input
      begin
         PUser^[Count]:=PUser^[Count]+1; // increase COUNT
         if PUser^[Count]>MaxCount then PUser^[Count]:=0; //check overflow
         For i:=0 to NumBits-1 do
         begin
            if (round(PUser^[Count]) and (1 shl i))>0 then POutput^[i]:=5 else POutput[i]:=0; //Set binary outputs with current COUNT
         end;
      end;
   end;
   PUser^[CLK_old]:=PInput^[CLK]; //Remember CLK status for next call
end;

Procedure Configure(PUser: PDLLParams);
var DLG: TCounterConfigDlg;
begin
   // call config. dialogue INITIAL VALUE and COUNTER BITS
   // PUser^[CounterBits]:
   // 0 = 4 bit counter
   // 1 = 8 bit conter
   // 2 = 16 bit counter

   DLG:=TCounterConfigDlg.Create(nil);                       //Create user dialogue
   With DLG do
   begin
      Caption:='Counter [DLL'+FloatToStr(PUser^[DLLIndex])+']'; // Show ProfiLab DLL-Index in dialogue caption

      ValueEdit.Text:=FloatToStr(PUser^[InitialCount]);      // Set dialogue values
      BitsCombo.ItemIndex:=round(PUser^[CounterBits]);       // Set dialogue values

      if ShowModal=mrOK then                                 // Execute dialogue
      begin
         PUser^[InitialCount]:=StrToInt(ValueEdit.Text);     // Read back values from dialogue
         PUser^[CounterBits]:=BitsCombo.ItemIndex;           // Read back values from dialogue;
      end;

      free;                                                  // Dont forget to free dialogue
   end;
end;


//export methods for ProfiLab
exports SimStart,
        SimStop,
        NumInputs,
        NumOutputsEx,
        Calculate,
        InputName,
        OutputName,
        Configure;
begin
end.

